package net.bonotown.android.aravadebo;

import net.bonotown.android.aravadebo.model.PosicioSomereta;
import net.bonotown.android.aravadebo.model.PostFoto;
import net.bonotown.android.aravadebo.model.PostText;
import net.bonotown.android.aravadebo.model.PostVideo;
import net.bonotown.android.aravadebo.model.gestio.ItemLlistaAgenda;
import net.bonotown.android.aravadebo.model.gestio.ItemLlistaHistoria;
import net.bonotown.android.aravadebo.model.gestio.ItemLlistaUbicacio;

import com.parse.Parse;
import com.parse.ParseACL;
import com.parse.ParseObject;
import com.parse.ParseUser;

import android.app.Application;
import android.content.Context;
import android.content.SharedPreferences;

public class AraVaDeBoApplication extends Application {

	// Key for saving the search distance preference
	private static final String CLAU_DISTANCIA_CERCA_POST = "distanciaCercaPost";
	private static final String CLAU_INTERVAL_ACTUALITZAR_POSICIO_SOMERETA = "actualitzarPosicioSomereta";
	
	private static SharedPreferences preferencies;

	public AraVaDeBoApplication() {}

	@Override
	public void onCreate() {
		super.onCreate();

		/*
		 * Registrem totes les classes que seran objectes a parse.com
		 */
		//Classe per mapeja l'objecte de parse.com que emmagatzema la posició de Sa Somereta
		ParseObject.registerSubclass(PosicioSomereta.class);

		//Classes per enviar post des d'una ubicació en el mapa
		ParseObject.registerSubclass(PostText.class);
		ParseObject.registerSubclass(PostFoto.class);
		ParseObject.registerSubclass(PostVideo.class);

		//Classes de gestió de les dades de l'aplicació
		ParseObject.registerSubclass(ItemLlistaUbicacio.class);
		ParseObject.registerSubclass(ItemLlistaAgenda.class);
		ParseObject.registerSubclass(ItemLlistaHistoria.class);

		//Inicialitzem la connexió de l'aplicació amb parse.com
		Parse.initialize(this, "enter_your_parse_server_key", "enter_your_parse_app_key");

		//Permetem usuari automàtic per qui no es registri o identifiqui
		ParseUser.enableAutomaticUser();

		//incrementem el comptador d'entrades a parse.com de l'usuari actual
		ParseUser.getCurrentUser().increment("RunCount");
		ParseUser.getCurrentUser().saveInBackground();

		//configuració per defecte dels usuaris a parse.com
		ParseACL defaultACL = new ParseACL();

		/*
		 * per defecte no permetrem que els usuaris anònims puguin llegir ni escriure dades
		 * després a cada objecte de gestió ja li indicarem si serà públic o no
		 */
		defaultACL.setPublicReadAccess(false);
		defaultACL.setPublicWriteAccess(false);

		ParseACL.setDefaultACL(defaultACL, true);
		
		preferencies = getSharedPreferences("net.bonotown.android.aravadebo", Context.MODE_PRIVATE);
	}

	public static long getDistanciaCercaPost() {
		return preferencies.getLong(CLAU_DISTANCIA_CERCA_POST, 250);
	}

	public static void setDistanciaCercaPost(long distancia) {
		preferencies.edit().putLong(CLAU_DISTANCIA_CERCA_POST, distancia).commit();
	}
	
	public static int getIntervalActualitzarPosicioSomereta() {
		return preferencies.getInt(CLAU_INTERVAL_ACTUALITZAR_POSICIO_SOMERETA, 10000);
	}

	public static void setIntervalActualitzarPosicioSomereta(int interval) {
		preferencies.edit().putInt(CLAU_INTERVAL_ACTUALITZAR_POSICIO_SOMERETA, interval).commit();
	}
}
