package net.bonotown.android.aravadebo;

import java.util.ArrayList;
import java.util.List;

import net.bonotown.android.aravadebo.model.PosicioSomereta;
import net.bonotown.android.aravadebo.model.PostFoto;
import net.bonotown.android.aravadebo.model.PostText;
import net.bonotown.android.aravadebo.model.PostVideo;
import net.bonotown.android.aravadebo.util.gps.GPSTracker;

import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GooglePlayServicesClient;
import com.google.android.gms.location.LocationListener;
import com.google.android.gms.maps.CameraUpdate;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.GoogleMap.OnMapClickListener;
import com.google.android.gms.maps.GoogleMap.OnMarkerClickListener;
import com.google.android.gms.maps.GoogleMap.OnMyLocationButtonClickListener;
import com.google.android.gms.maps.MapView;
import com.google.android.gms.maps.MapsInitializer;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.GroundOverlay;
import com.google.android.gms.maps.model.GroundOverlayOptions;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;
import com.parse.FindCallback;
import com.parse.ParseAnonymousUtils;
import com.parse.ParseException;
import com.parse.ParseFile;
import com.parse.ParseGeoPoint;
import com.parse.ParseQuery;
import com.parse.ParseUser;

import android.app.Activity;
import android.content.Intent;
import android.content.IntentSender;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.location.Location;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.os.SystemClock;
import android.support.v4.app.Fragment;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import android.widget.Toast;

public class FragmentMap extends Fragment implements LocationListener,
GooglePlayServicesClient.ConnectionCallbacks,
GooglePlayServicesClient.OnConnectionFailedListener,
OnMyLocationButtonClickListener,
OnMarkerClickListener,OnMapClickListener {

	/*
	 * Define a request code to send to Google Play services This code is returned in
	 * Activity.onActivityResult
	 */
	private final static int CONNECTION_FAILURE_RESOLUTION_REQUEST = 9000;

	// Conversion from feet to meters
	private static final float METERS_PER_FEET = 0.3048f;

	// Conversion from kilometers to meters
	private static final int METERS_PER_KILOMETER = 1000;

	//variables de control sobre objectes
	private MapView mapView;
	private GoogleMap mapaSomereta;

	//inicialitzem l'array de markers per les diferents ubicacions existents
	private List<MarkerOptions> markersUbicacions = new ArrayList<MarkerOptions>(); //marcadors d'ubicacions disponibles
	private List<Marker> markersControlUbicacions = new ArrayList<Marker>(); //llista de marcadors d'ubicacions visibles en el mapa

	private List<Marker> markersPostText = new ArrayList<Marker>(); //llista de marcadors de posts text en el mapa
	private List<Marker> markersPostFoto = new ArrayList<Marker>(); //llista de marcadors de posts foto en el mapa
	private List<Marker> markersPostVideo = new ArrayList<Marker>(); //llista de marcadors de posst video en el mapa

	private CarregarPostMapaTask mTask;

	protected ArrayList<PostFoto> llistaPostFoto = new ArrayList<PostFoto>(); // llista amb totes les URL per veure imatges i videos
	protected int posicio = 0;
	protected GPSTracker gpsTracker;
	protected Handler someretaHandler;
	protected Marker markerSomereta; //marcador en el mapa de Sa Somereta
	protected float radiCerca;
	protected GroundOverlay imageOverlay = null;

	//fil d'execució per anar actualitzant la posició de Sa Somereta
	private Runnable posicionadorSomererta = new Runnable() {

		@Override 
		public void run() {
			try {
				posicionarSomereta();
			} catch (ParseException e) {
				Log.e(getActivity().getLocalClassName(), "Problemes en llegir posició de Sa Somereta - " + e.getMessage());
			}
			someretaHandler.postDelayed(posicionadorSomererta, AraVaDeBoApplication.getIntervalActualitzarPosicioSomereta());
		}
	};

	// constructor crida't des de l'activitat principal
	public FragmentMap(){}

	@Override
	public void onCreate(Bundle savedInstanceState){
		super.onCreate(savedInstanceState);

		someretaHandler = new Handler(); //inicialitzem el handler per refrescar la posició de Sa Somereta

		radiCerca = AraVaDeBoApplication.getDistanciaCercaPost(); //distancia per mostra els posts		
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {

		View rootView = inflater.inflate(R.layout.fragment_map, container, false);

		mapView = (MapView) rootView.findViewById(R.id.mapa_somereta);
		mapView.onCreate(savedInstanceState);

		return rootView;
	}

	@Override
	public void onViewCreated(View view, Bundle savedInstanceState) {
		// assignem el mapa de google a la variable que el controla
		mapaSomereta = mapView.getMap();

		if (mapaSomereta == null) {
			Toast.makeText(getActivity(), "Ho sentim però no es pot carregar el mapa. Prova-ho de nou més tard.", Toast.LENGTH_LONG ).show();
			getActivity().finish();
		} else {
			// creem controlador de localització
			gpsTracker = new GPSTracker(getActivity());

			//inicialitzem el mapa
			MapsInitializer.initialize(getActivity());

			mapaSomereta.setMapType(GoogleMap.MAP_TYPE_NORMAL);

			try {
				posicionarSomereta();
				zoomSomereta(); //fem zoom damunt la seva posició
			} catch (ParseException e1) {
				Log.e(getActivity().getClass().getName(),"Error en accedir a la posició actual de somereta: " + e1.getMessage());
			}

			startActualitzacioPosicioSomereta(); //iniciem la tasca que anirà actualitzant posició Somereta

			if (! ParseAnonymousUtils.isLinked(ParseUser.getCurrentUser())) {
				// mostrem la barra d'eines per compartir multimedia i text
				inicialitzarBarraEinesCompartir(view);

				//si és usuari registrat permetem que pugui veure la seva posició
				mapaSomereta.setMyLocationEnabled(true);
				
				mapaSomereta.setOnMyLocationButtonClickListener(new OnMyLocationButtonClickListener() {

					@Override
					public boolean onMyLocationButtonClick() {

						if (mapaSomereta.getMapType()==GoogleMap.MAP_TYPE_NONE) {
							imageOverlay.remove();
							mapaSomereta.setMapType(GoogleMap.MAP_TYPE_NORMAL);
						}

						return false;
					}
				});
				
				mapaSomereta.setOnMapClickListener(new OnMapClickListener() {
					
					@Override
					public void onMapClick(LatLng point) {
						if (mapaSomereta.getMapType()==GoogleMap.MAP_TYPE_NONE) {
							imageOverlay.remove();
							mapaSomereta.setMapType(GoogleMap.MAP_TYPE_NORMAL);
						}
					}
				});
				
				mapaSomereta.setOnMarkerClickListener(new OnMarkerClickListener() {

					@Override
					public boolean onMarkerClick(Marker marker) {
						try {
							if (mapaSomereta.getMapType()==GoogleMap.MAP_TYPE_NONE) {
								mapaSomereta.setMapType(GoogleMap.MAP_TYPE_NORMAL);
								imageOverlay.remove();
							}
							try{
								int num = Integer.parseInt(marker.getTitle());
								
								mapaSomereta.setMapType(GoogleMap.MAP_TYPE_NONE);
								CameraUpdate camera = CameraUpdateFactory.newLatLngZoom(marker.getPosition(), 17.0f);
								mapaSomereta.animateCamera(camera);

								PostFoto foto = llistaPostFoto.get(num);
								ParseFile file = foto.getFoto();
								Bitmap image = BitmapFactory.decodeByteArray(file.getData(), 0, file.getData().length);

								GroundOverlayOptions newarkMap = new GroundOverlayOptions()
								.image(BitmapDescriptorFactory.fromBitmap(image))
								.anchor(0, 1)
								.position(marker.getPosition(), 220f);

								imageOverlay = mapaSomereta.addGroundOverlay(newarkMap);
							} catch (NumberFormatException e) {
								mapaSomereta.setMapType(GoogleMap.MAP_TYPE_NORMAL);
							}
						} catch (ParseException e) {
							mapaSomereta.setMapType(GoogleMap.MAP_TYPE_NORMAL);
						}

						return true;
					}
				});

				// tasca en background per anar actualitzant els posts en el mapa segons el radi damunt posició
				mTask = new CarregarPostMapaTask();
				mTask.execute();
			} else {
				//si no es un usuari registrat a l'aplicació no permetem que cerqui la seva ubicació
				mapaSomereta.setMyLocationEnabled(false);
			}
		}
	}

	@Override
	public void onSaveInstanceState(Bundle outState) {
		super.onSaveInstanceState(outState);

		outState.putAll(outState);
	}

	@Override
	public void onDestroyView() {
		stopActualitzacioPosicioSomereta();
		super.onDestroyView();
	}

	/**
	 * Hold a reference to the parent Activity so we can report the
	 * task's current progress and results. The Android framework 
	 * will pass us a reference to the newly created Activity after 
	 * each configuration change.
	 */
	@Override
	public void onAttach(Activity activity) {
		super.onAttach(activity);
	}

	/**
	 * Set the callback to null so we don't accidentally leak the 
	 * Activity instance.
	 */
	@Override
	public void onDetach() {
		super.onDetach();
	}

	@Override
	public void onResume() {
		super.onResume();
		radiCerca = AraVaDeBoApplication.getDistanciaCercaPost();
		mapView.onResume();
	}

	@Override
	public void onPause() {
		super.onPause();
		mapView.onPause();
	}

	@Override
	public void onDestroy() {
		super.onDestroy();
		mapView.onDestroy();
	}

	@Override
	public void onLowMemory() {
		super.onLowMemory();
		mapView.onLowMemory();
	}

	@Override
	public void onConnectionFailed(ConnectionResult connectionResult) {
		if (connectionResult.hasResolution()) {
			try {

				// Start an Activity that tries to resolve the error
				connectionResult.startResolutionForResult(getActivity(), CONNECTION_FAILURE_RESOLUTION_REQUEST);

			} catch (IntentSender.SendIntentException e) {
				Log.e(getActivity().getLocalClassName(), "Error durant la connexió als serveis de mapes de Google", e);
			}
		} else {
			Toast.makeText(getActivity(), "Error: " + connectionResult.getErrorCode(), Toast.LENGTH_LONG).show();
		}
	}

	@Override
	public void onConnected(Bundle connectionHint) {
		mTask.execute();
	}

	@Override
	public void onDisconnected() {
		mTask.cancel(true);

	}

	@Override
	public void onLocationChanged(Location location) {
		// TODO Auto-generated method stub

	}

	@Override
	public boolean onMarkerClick(Marker marker) {
		// TODO Auto-generated method stub
		return false;
	}

	//inicia l'execució del fil per actualitzar la posicó de Sa Somereta
	private void startActualitzacioPosicioSomereta() {
		posicionadorSomererta.run(); 
	}

	//atura l'execució del fil per actualitzar la posicó de Sa Somereta
	private void stopActualitzacioPosicioSomereta() {
		someretaHandler.removeCallbacks(posicionadorSomererta);
	}

	private void inicialitzarBarraEinesCompartir(View view) {
		LinearLayout barra_eines_compartir = (LinearLayout) view.findViewById(R.id.barra_eines_compartir);

		barra_eines_compartir.setVisibility(View.VISIBLE); //fem visible la barra d'eines

		//boto per fer el zoom damunt la posició de Sa Somereta
		ImageButton boto = (ImageButton) barra_eines_compartir.findViewById(R.id.zoom_somereta);
		boto.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				if (mapaSomereta.getMapType()==GoogleMap.MAP_TYPE_NONE) {
					mapaSomereta.setMapType(GoogleMap.MAP_TYPE_NORMAL);
					imageOverlay.remove();
				}
				zoomSomereta();
			}
		});

		//botó per pujar post de text
		boto = (ImageButton) barra_eines_compartir.findViewById(R.id.boto_pujar_text);
		boto.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				if (gpsTracker.canGetLocation()) {
					Location ubicacio_actual =  gpsTracker.getLocation(); //agafa la posició actual del terminal
					Intent intent = new Intent(getActivity(), PostTextActivity.class);
					intent.putExtra("latitude", ubicacio_actual.getLatitude()); //passem la latitud a l'activitat
					intent.putExtra("longitude", ubicacio_actual.getLongitude());//passem la longitud a l'activitat
					startActivity(intent);
				} else {
					Toast.makeText(getActivity(), "No es pot determinar la posicó, reviseu configuració d'ubicació del terminal.", Toast.LENGTH_LONG).show();
				}
			}
		});

		boto = (ImageButton) barra_eines_compartir.findViewById(R.id.boto_pujar_imatge);
		boto.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				if (gpsTracker.canGetLocation()) {
					Location ubicacio_actual =  gpsTracker.getLocation(); //agafa la posició actual del terminal
					Intent intent = new Intent(getActivity(), CameraActivity.class);
					intent.putExtra("latitude", ubicacio_actual.getLatitude()); //passem la latitud a l'activitat
					intent.putExtra("longitude", ubicacio_actual.getLongitude());//passem la longitud a l'activitat
					intent.putExtra("esVideo", false); //indiquem a l'activitat que s'obrira que es vol fer una foto
					startActivity(intent);
				} else {
					Toast.makeText(getActivity(), "No es pot determinar la posicó, reviseu configuració d'ubicació del terminal.", Toast.LENGTH_LONG).show();
				}
			}
		});

		boto = (ImageButton) barra_eines_compartir.findViewById(R.id.boto_pujar_video);
		boto.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				if (gpsTracker.canGetLocation()) {
					Location ubicacio_actual =  gpsTracker.getLocation(); //agafa la posició actual del terminal
					Intent intent = new Intent(getActivity(), CameraActivity.class);
					intent.putExtra("latitude", ubicacio_actual.getLatitude()); //passem la latitud a l'activitat
					intent.putExtra("longitude", ubicacio_actual.getLongitude());//passem la longitud a l'activitat
					intent.putExtra("esVideo", true); //indiquem a l'activitat que s'obrira que es vol fer un video
					startActivity(intent);
				} else {
					Toast.makeText(getActivity(), "No es pot determinar la posicó, reviseu configuració d'ubicació del terminal.", Toast.LENGTH_LONG).show();
				}
			}
		});
	}

	/*
	 * mètode public per comunicar-se amb l'activitat:
	 * quan carreguem la llista d'ubicacions en el drawer dret, creem un marker en el mapa
	 */
	public void crearMarker(LatLng latLng, String titol) {
		MarkerOptions marker = new MarkerOptions();
		//marker.icon(BitmapDescriptorFactory.fromResource(R.drawable.marker));

		marker.position(latLng);
		marker.title(titol);

		marker.draggable(false);
		marker.visible(true);

		markersUbicacions.add(marker);
	}

	/*
	 * mètode public per comunicar-se amb l'activitat:
	 * Ens servira per mostrar o amagar en el mapa les ubicacions d'interés
	 * en el cas que no hi hagi cap farà el zoom damunt de la posició de Sa Somereta
	 */
	public void posicionarUbicacio(int posicio, boolean seleccionat) {
		Marker marker = null;
		
		if (mapaSomereta.getMapType()==GoogleMap.MAP_TYPE_NONE) {
			mapaSomereta.setMapType(GoogleMap.MAP_TYPE_NORMAL);
			imageOverlay.remove();
		}
		
		if (seleccionat) {
			marker = mapaSomereta.addMarker(markersUbicacions.get(posicio));
			marker.showInfoWindow();
			markersControlUbicacions.add(marker);
		} else {
			marker = markersControlUbicacions.get(posicio);
			marker.remove();
			markersControlUbicacions.remove(posicio);

			if (markersControlUbicacions.size()==0) { //si no queda cap marker a la llista
				marker = markerSomereta; //agafem el marker de sa somereta
			} else {
				marker = markersControlUbicacions.get(markersControlUbicacions.size()-1);
			}
		}

		if (marker != null) {
			CameraUpdate camera = CameraUpdateFactory.newLatLngZoom(marker.getPosition(), 15.0f);
			mapaSomereta.animateCamera(camera);
		}
	}

	protected void mostrarPostTextMapa() throws ParseException {
		ParseQuery<PostText> query = PostText.getQuery();

		query.whereWithinKilometers("localitzacio", transformarParseGeoPoint(gpsTracker.getLocation()), radiCerca * METERS_PER_FEET	/ METERS_PER_KILOMETER );

		query.findInBackground(new FindCallback<PostText>() {

			@Override
			public void done(List<PostText> llista_post_text, ParseException e) {
				if (e==null) {
					for (PostText postText : llista_post_text) {
						//Marcador per cadascun dels post de comentaris de text
						MarkerOptions markerOptions = new MarkerOptions();

						markerOptions.icon(BitmapDescriptorFactory.fromResource(R.drawable.ic_action_pujar_comentari));

						markerOptions.position(new LatLng(postText.getParseGeoPoint("localitzacio").getLatitude(), 
								postText.getParseGeoPoint("localitzacio").getLongitude()));

						markerOptions.title(postText.getComentari());
						markerOptions.draggable(false);
						markerOptions.visible(true);

						mapaSomereta.addMarker(markerOptions);

						markersPostText.add(mapaSomereta.addMarker(markerOptions));
					}
				} else {
					Log.e(getActivity().getClass().getName(), "Mostrar text: " + e.getMessage());
				}
			}
		});
	}

	protected void mostrarPostFotoMapa() throws ParseException {
		ParseQuery<PostFoto> query = PostFoto.getQuery();

		query.whereWithinKilometers("localitzacio", transformarParseGeoPoint(gpsTracker.getLocation()), radiCerca * METERS_PER_FEET	/ METERS_PER_KILOMETER );

		query.findInBackground(new FindCallback<PostFoto>() {

			@Override
			public void done(List<PostFoto> llista_post_foto, ParseException e) {
				if (e==null) {
					for (PostFoto postFoto : llista_post_foto) {
						//Marcador per cadascun dels post de comentaris de text
						MarkerOptions markerOptions = new MarkerOptions();

						markerOptions.position(new LatLng(postFoto.getParseGeoPoint("localitzacio").getLatitude(), 
								postFoto.getParseGeoPoint("localitzacio").getLongitude()));

						markerOptions.icon(BitmapDescriptorFactory.fromResource(R.drawable.ic_action_pujar_imatge));
						markerOptions.title(String.valueOf(posicio));
						markerOptions.infoWindowAnchor(0, 1);
						markerOptions.draggable(false);
						markerOptions.visible(true);

						llistaPostFoto.add(postFoto);
						posicio++;

						mapaSomereta.addMarker(markerOptions);

						markersPostFoto.add(mapaSomereta.addMarker(markerOptions));
					}
				} else {
					Log.e(getActivity().getClass().getName(), "Mostrar fotos: " + e.getMessage());
				}
			}
		});
	}

	protected void mostrarPostVideoMapa() throws ParseException {
		ParseQuery<PostVideo> query = PostVideo.getQuery();

		query.whereWithinKilometers("localitzacio", transformarParseGeoPoint(gpsTracker.getLocation()), radiCerca * METERS_PER_FEET	/ METERS_PER_KILOMETER );

		query.findInBackground(new FindCallback<PostVideo>() {

			@Override
			public void done(List<PostVideo> llista_post_video, ParseException e) {
				if (e==null) {
					for (PostVideo postVideo : llista_post_video) {
						//Marcador per cadascun dels post de videos
						MarkerOptions markerOptions = new MarkerOptions();

						markerOptions.position(new LatLng(postVideo.getParseGeoPoint("localitzacio").getLatitude(), 
								postVideo.getParseGeoPoint("localitzacio").getLongitude()));

						Bitmap bmp = BitmapFactory.decodeResource(getResources(), R.drawable.ic_action_pujar_video);
						markerOptions.icon(BitmapDescriptorFactory.fromBitmap(Bitmap.createScaledBitmap(bmp, 50, 50, false)));

						//llistaUrlPost = append(llistaUrlPost, postVideo.getVideo().getUrl());

						markerOptions.title(postVideo.getVideo().getUrl());
						markerOptions.draggable(false);
						markerOptions.visible(true);

						mapaSomereta.addMarker(markerOptions);

						markersPostVideo.add(mapaSomereta.addMarker(markerOptions));
					}
				} else {
					Log.e(getActivity().getClass().getName(), "Mostrar videos: " + e.getMessage());
				}
			}
		});
	}

	//Llegeix la posició de Sa Somereta
	private void posicionarSomereta() throws ParseException {

		PosicioSomereta posicioSomereta = PosicioSomereta.getQuery().getFirst();
		if (posicioSomereta != null) {
			//Marcador per la posició de Sa Somereta
			MarkerOptions markerOptions = new MarkerOptions();
			markerOptions.icon(BitmapDescriptorFactory.fromResource(R.drawable.ic_somereta));
			markerOptions.position(new LatLng(posicioSomereta.getParseGeoPoint("localitzacio").getLatitude(), 
					posicioSomereta.getParseGeoPoint("localitzacio").getLongitude()));
			markerOptions.title(getResources().getString(R.string.titol_marker_somereta));
			markerOptions.draggable(false);
			markerOptions.visible(true);

			if (markerSomereta!=null) markerSomereta.remove(); //esborrem la darrera posició
			markerSomereta = mapaSomereta.addMarker(markerOptions);
		} else {
			Log.e(getActivity().getClass().getName(), "Impossible recuperar posició Somereta");
		}
	}

	private ParseGeoPoint transformarParseGeoPoint(Location loc) {
		return new ParseGeoPoint(loc.getLatitude(), loc.getLongitude());
	}

	private void zoomSomereta() {
		if (markerSomereta != null) { //si hi ha posició marcada farem un zoom
			CameraUpdate camera = CameraUpdateFactory.newLatLngZoom(markerSomereta.getPosition(), 15.0f);
			mapaSomereta.animateCamera(camera);
		}
	}

	private class CarregarPostMapaTask extends AsyncTask<Void, Integer, Void> {

		@Override
		protected void onPreExecute() {}

		@Override
		protected Void doInBackground(Void... ignore) {
			while(!isCancelled()) {
				try {
					if (gpsTracker.canGetLocation()) {
						llistaPostFoto = new ArrayList<PostFoto>();
						posicio = 0;
						mostrarPostTextMapa();
						mostrarPostFotoMapa();
						mostrarPostVideoMapa();
					}
				} catch (ParseException e) {
					Log.e(getActivity().getLocalClassName(), "Error en la càrrega de dades - " + e.getMessage());
				}
				SystemClock.sleep(20000);
			}
			return null;
		}

		@Override
		protected void onProgressUpdate(Integer... percent) {
		}

		@Override
		protected void onCancelled() {
		}

		@Override
		protected void onPostExecute(Void ignore) {
		}
	}

	@Override
	public boolean onMyLocationButtonClick() {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public void onMapClick(LatLng point) {
		// TODO Auto-generated method stub
		
	}	
}
