package net.bonotown.android.aravadebo;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import com.google.android.gms.maps.model.LatLng;
import com.parse.FindCallback;
import com.parse.ParseAnalytics;
import com.parse.ParseAnonymousUtils;
import com.parse.ParseException;
import com.parse.ParseInstallation;
import com.parse.ParseUser;
import com.parse.PushService;

import android.content.res.Configuration;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.os.Bundle;
import android.preference.PreferenceFragment;
import android.support.v4.app.ActionBarDrawerToggle;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.FragmentManager;
import android.support.v4.widget.DrawerLayout;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.CheckBox;
import android.widget.ListView;
import net.bonotown.android.aravadebo.adapter.LlistaUbicacioAdapter;
import net.bonotown.android.aravadebo.adapter.NavDrawerListAdapter;
import net.bonotown.android.aravadebo.model.NavDrawerButtons;
import net.bonotown.android.aravadebo.model.NavDrawerButtonsAuthUser;
import net.bonotown.android.aravadebo.model.NavDrawerItem;
import net.bonotown.android.aravadebo.model.gestio.ItemLlistaUbicacio;

public class MainActivity extends FragmentActivity {

	//llista de fragments disponibles
	public static List<Object> listFragments;

	private DrawerLayout mDrawerLayout;
	private ListView mDrawerList;
	private ListView mDrawerUbicacions;
	private ActionBarDrawerToggle mDrawerToggle;

	// Títol del drawer pel menú principal
	private CharSequence mDrawerTitle;

	// Títol de l'aplicació
	private CharSequence mTitle;

	// slider menu items details
	private String[] navMenuTitles;

	// llista d'ítems de diferent naturalesa pel drawer
	private ArrayList<Object> navDrawerItems;

	// adaptadors pels dos menús lliscants
	private NavDrawerListAdapter adapter; //menú principal
	private LlistaUbicacioAdapter adapterUbicacions; //menú d'ubicacions interés

	// mètode per poder afegir ítems en un array tipus [] 
	static <T> T[] append(T[] arr, T element) {
		final int N = arr.length;
		arr = Arrays.copyOf(arr, N + 1);
		arr[N] = element;
		return arr;
	}

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		//Si és una nova instal·lació quedarà registrada a parse.com
		PushService.setDefaultPushCallback(this, MainActivity.class);
		ParseInstallation.getCurrentInstallation().saveInBackground();

		//Enviem estadística d'aplicació oberta
		ParseAnalytics.trackAppOpened(getIntent());

		//Obrim el layout d'aquesta activitat
		setContentView(R.layout.activity_main);

		//idetifinca el títol de l'activitat i del menú
		mTitle = mDrawerTitle = getTitle();

		// inicialitzem la llista de fragments disponibles des de la pantalla principal
		listFragments = new ArrayList<Object>();
		listFragments.add(new FragmentMap());
		listFragments.add(new FragmentAgenda());
		listFragments.add(new FragmentHistoria());

		// Enable action bar icon_luncher as toggle Home Button
		getActionBar().setDisplayHomeAsUpEnabled(true);

		try {
			getActionBar().setHomeButtonEnabled(true);
		} catch (Exception e) {
			// Not available in versions < 14
		}
	}

	@Override
	protected void onStart() {
		super.onStart();

		if(mDrawerLayout == null || mDrawerList == null || mDrawerUbicacions == null || mDrawerToggle == null) {
			//instanciem el menús principal i d'ubicacions
			mDrawerLayout = (DrawerLayout) findViewById(R.id.drawer_layout);
			mDrawerList = (ListView) findViewById(R.id.list_slidermenu);
			mDrawerUbicacions = (ListView) findViewById(R.id.list_sliderubicacion);

			// Creem l'objecte que contindrà els ítems del menú principal
			navDrawerItems = new ArrayList<Object>();

			// Afegim cadascún dels ítems del menú principal
			navMenuTitles = getResources().getStringArray(R.array.nav_drawer_items);
			for (String itemTitle : navMenuTitles) {
				navDrawerItems.add(new NavDrawerItem(itemTitle));
			}

			if (! ParseAnonymousUtils.isLinked(ParseUser.getCurrentUser())) {

				//Afegim item per a la gestió en cas que sigui l'administrador de l'aplicació 
				if (ParseUser.getCurrentUser().getUsername().compareTo("bonotown")==0) {
					listFragments.add(new FragmentGestio());
					navDrawerItems.add(new NavDrawerItem(getResources().getString(R.string.titol_gestio)));
					navMenuTitles = append(navMenuTitles, getResources().getString(R.string.titol_gestio));
				}
				
				// Afegim els botons d'usuari autenticat en el drawer principal
				navDrawerItems.add(new NavDrawerButtonsAuthUser(MainActivity.this));

				// canviem el color de la barra al color d'usuari autenticat
				getActionBar().setBackgroundDrawable(new ColorDrawable(Color.rgb(1, 155, 15)));
			} else {
				// Afegim els botons per a usuaris anònims
				navDrawerItems.add(new NavDrawerButtons(MainActivity.this));

				// canviem el color de la barra al color d'usuari sense autenticar
				getActionBar().setBackgroundDrawable(new ColorDrawable(Color.rgb(0, 0, 0)));
			}

			// Creem l'adaptador per al menú principal
			adapter = new NavDrawerListAdapter(MainActivity.this, navDrawerItems);

			//Escoltarà qualsevol clic damunt items de la llita del menú
			mDrawerList.setOnItemClickListener(new SlideMenuClickListener());
			mDrawerList.setAdapter(adapter);

			// carregar llista ubicacions interes
			omplirLlistaUbicacions();

			//creem el llençador i controlador pel drawer
			mDrawerToggle = new ActionBarDrawerToggle(this, mDrawerLayout, R.drawable.ic_drawer, R.string.app_name, R.string.app_name) {

				public void onDrawerClosed(View drawerView) {
					if(drawerView.equals(mDrawerList)) {
						getActionBar().setTitle(mTitle);
						supportInvalidateOptionsMenu();
						mDrawerToggle.syncState();
					}
				}

				public void onDrawerOpened(View drawerView) {
					if(drawerView.equals(mDrawerList)) {
						getActionBar().setTitle(mDrawerTitle);
						supportInvalidateOptionsMenu();
						mDrawerToggle.syncState();
					}
				}

				@Override
				public void onDrawerSlide(View drawerView, float slideOffset) {
					/*
					 * Comentant la linia següent, s'evita el comportament normal 
					 * de l'indicador del menú principal quan s'obre el menú d'ubicacions
					 */
					// super.onDrawerSlide(drawerView, slideOffset);
				}
			};

			mDrawerLayout.setDrawerListener(mDrawerToggle);
		}
	}

	@Override
	protected void onPostCreate(Bundle savedInstanceState) {
		super.onPostCreate(savedInstanceState);

		mDrawerToggle.syncState();

		if (savedInstanceState == null) { 
			displayView(0); //D'entrada mostrem el fragment amb el mapa
		}
	}

	@Override
	protected void onResume() {
		super.onResume();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}

	@Override
	public boolean onPrepareOptionsMenu(Menu menu) {
		return super.onPrepareOptionsMenu(menu);
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		//si el ítem és el del menú principal 
		if (mDrawerToggle.onOptionsItemSelected(item)) {
			if (mDrawerLayout.isDrawerOpen(mDrawerUbicacions)) {
				mDrawerLayout.closeDrawer(mDrawerUbicacions); //tancarem el drawer d'ubicacions si estava obert
			}

			return true;
		}

		switch (item.getItemId()) {
		case R.id.accio_ubicacions: //si el ítem és el de la llista d'ubicacions d'interés

			if(mDrawerLayout.isDrawerOpen(mDrawerUbicacions)) {
				mDrawerLayout.closeDrawer(mDrawerUbicacions); //si estava obert el tanquem
			} else {
				mDrawerLayout.openDrawer(mDrawerUbicacions); //sino l'obrim
				if (mDrawerLayout.isDrawerOpen(mDrawerList)) {
					mDrawerLayout.closeDrawer(mDrawerList); // i taquem el menú principal si estava obert
				}
			}

		default:
			return super.onOptionsItemSelected(item);
		}
	}

	@Override
	public void setTitle(CharSequence title) {
		mTitle = title;
		getActionBar().setTitle(mTitle);
		mDrawerToggle.syncState();
	}

	@Override
	public void onConfigurationChanged(Configuration newConfig) {
		super.onConfigurationChanged(newConfig);
		mDrawerToggle.onConfigurationChanged(newConfig);
	}

	// Escolta els clics en el menú principal i mostra el fragment corresponent si és de tipus ítem del menú 
	private class SlideMenuClickListener implements ListView.OnItemClickListener {

		@Override
		public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
			if (parent.getAdapter().getItemViewType(position)==0) {
				// si la posició és diferent a l'ítem per mostrar el mapa ocultarem el drawer d'ubicacions
				if (position>0) {

				}

				displayView(position); //mostrem el fragment corresponent
			} else {
				//si són els botons d'usuari no cal fer res aquí
			}
		}
	}

	// Escolta els clics a la llista d'ubicacions d'interes del drawer de la dreta
	private class UbicacionsMenuClickListener implements ListView.OnItemClickListener {

		@Override
		public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
			CheckBox seleccio = (CheckBox) view.findViewById(R.id.item_ubicacio_seleccionat);
			seleccio.setChecked(! seleccio.isChecked()); //selecciona si desmarcat -- deselecciona si marcat

			// Agafem el fragment del mapa per mostra o amagar la ubicació d'interés seleccionada 
			FragmentMap fragment = (FragmentMap) listFragments.get(0);

			// cridem la funció per comunicar activitat amb fragment
			fragment.posicionarUbicacio(position, seleccio.isChecked());

			// marquem o desmarquem el ítem de la llista
			mDrawerUbicacions.setItemChecked(position, seleccio.isChecked());
			mDrawerUbicacions.setSelection(position);

			//tanquem el drawer d'ubicacions d'interés
			mDrawerLayout.closeDrawer(mDrawerUbicacions);
		}
	}

	//Mostrarà el fragment pertinent depenent de la selecció del menú principal
	private void displayView(int position) {
		FragmentManager fragmentManager = getSupportFragmentManager();
		android.app.FragmentManager fragmentManagerPref = getFragmentManager();
		
		if (listFragments.get(position) instanceof Fragment) {
			Fragment fragment = (Fragment) listFragments.get(position); //obtenim el fragment
			fragmentManager.beginTransaction().replace(R.id.frame_container, fragment).commit();
			
			// ocultarà el drawer d'ubicacions si no és el fragment del mapa
			if(fragment instanceof FragmentMap) {
				try {
					mDrawerLayout.addView(mDrawerUbicacions);
				} catch(Exception ignore) {}
			} else {
				mDrawerLayout.removeView(mDrawerUbicacions);
			}
		} else {
			PreferenceFragment fragment_pref = (PreferenceFragment) listFragments.get(position); //obtenim el fragment
			fragmentManagerPref.beginTransaction().replace(R.id.frame_container, fragment_pref).commit();
			
			mDrawerLayout.removeView(mDrawerUbicacions); //ocultem el drawer d'ubicacions
		}
		
		setTitle(navMenuTitles[position]); //indiquem el títol a la barra principal

		//marquem el ítem com a seleccionat
		mDrawerList.setItemChecked(position, true);
		mDrawerList.setSelection(position);
		mDrawerLayout.closeDrawer(mDrawerList);

		
	}

	private void omplirLlistaUbicacions() {
		//Carrega d'ubicacions d'interes
		ItemLlistaUbicacio.getQuery().findInBackground(new FindCallback<ItemLlistaUbicacio>() {

			@Override
			public void done(List<ItemLlistaUbicacio> llista_ubicacions, ParseException e) {
				if (e==null) {
					// Creem l'adaptador per al menú d'ubicacions interés
					adapterUbicacions = new LlistaUbicacioAdapter(MainActivity.this, R.layout.list_item_ubicacio, llista_ubicacions, true);

					//Escoltarà qualsevol clic damunt items de la llita d'ubicacions
					mDrawerUbicacions.setOnItemClickListener(new UbicacionsMenuClickListener());
					mDrawerUbicacions.setAdapter(adapterUbicacions);

					//Creem els markers per posicionar-los en el mapa
					for (ItemLlistaUbicacio ubicacio : llista_ubicacions) {
						//cridem la funció del FragmentMap que crea el marker
						((FragmentMap) listFragments.get(0)).crearMarker(
								new LatLng(ubicacio.getLatitude(),ubicacio.getLongitude()), ubicacio.getNom()); 
					}
				}
			}
		});
	}
}
