package net.bonotown.android.aravadebo;

import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.Date;

import net.bonotown.android.aravadebo.model.PostVideo;

import com.parse.ParseException;
import com.parse.ParseFile;
import com.parse.ParseGeoPoint;
import com.parse.ParseUser;
import com.parse.SaveCallback;

import android.app.Activity;
import android.content.ContentResolver;
import android.net.Uri;
import android.os.Bundle;
import android.support.v4.app.NavUtils;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.webkit.MimeTypeMap;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.VideoView;

public class PostVideoActivity extends Activity implements OnClickListener {

	//Declaració de variables a fer servir
	private PostVideo parseVideoPost;
	private double latitut;
	private double longitut;
	private Uri videoPost;
	private ParseFile videoFile;
	private TextView text_video_post;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		setContentView(R.layout.activity_post_video);
		setupActionBar();

		VideoView videoView = (VideoView) findViewById(R.id.video_post);
		
		text_video_post = (TextView) findViewById(R.id.text_video_post);
		
		latitut = getIntent().getDoubleExtra("latitude", 40.0010061);
		longitut = getIntent().getDoubleExtra("longitude", 3.8365123);

		videoPost = getIntent().getData();
		
		if (videoPost != null) {
			videoView.setVideoURI(videoPost);
			videoView.start();
			
			Button sendButton = (Button) findViewById(R.id.boto_enviar_video_post);
			sendButton.setOnClickListener(new OnClickListener() {

				@Override
				public void onClick(View v) {
					try {
						ContentResolver cr = getContentResolver();
						InputStream is = cr.openInputStream(videoPost);
						ByteArrayOutputStream bos = new ByteArrayOutputStream();
						byte[] bytes = new byte[3072];
						
						try {
							for (int readNum; (readNum = is.read(bytes)) != -1;) {
								bos.write(bytes, 0, readNum);
							}
						} catch (IOException e) {
							setResult(RESULT_CANCELED);
							Log.e(PostVideoActivity.class.getName(), "No s'ha pogut transformar el video. " + e.getMessage());
							finish();
						}
						
						//recuperem els bytes del video per poder-ho passar a parse.com
						byte[] data = bos.toByteArray();

						try {
							bos.close();
						} catch (IOException e) {
							Log.e(PostVideoActivity.class.getName(), e.getMessage());
						}

						String timeStamp = new SimpleDateFormat("yyyyMMdd_HHmmss").format(new Date());
						
						//Recuperem l'extensió des del MIMETYPE de l'arxiu.
						MimeTypeMap mime = MimeTypeMap.getSingleton();
						String videoExt = mime.getExtensionFromMimeType(cr.getType(videoPost));
						
						//objecte necessari per enviar arxiu a parse.com
						videoFile = new ParseFile(ParseUser.getCurrentUser() + "_vid_" + timeStamp + "." + videoExt, data);
						
						//desem en rerefons el video a parse.com
						videoFile.saveInBackground(new SaveCallback() {
							
							@Override
							public void done(ParseException e) {
								if(e == null) {
									//Creem el punt de coordenades on s'ha registrat la foto ParseGeoPoint
									ParseGeoPoint geoPoint = new ParseGeoPoint();
									geoPoint.setLatitude(latitut);
									geoPoint.setLongitude(longitut);
									
									//Objecte per emmagatzemar el video a parse.com
									parseVideoPost = new PostVideo();
									parseVideoPost.setLocation(geoPoint);
									parseVideoPost.setComentari(text_video_post.getText().toString());
									parseVideoPost.setUser(ParseUser.getCurrentUser());
									parseVideoPost.setVideo(videoFile);
									
									// Desem el post amb referencia al video, al thumbnail
									parseVideoPost.saveInBackground(new SaveCallback() {
										@Override
										public void done(ParseException e) {
											if(e == null) {
												setResult(RESULT_OK);
												Toast.makeText(PostVideoActivity.this, "Video enviat correctament", Toast.LENGTH_SHORT).show();
											} else {
												setResult(RESULT_CANCELED);
												Toast.makeText(PostVideoActivity.this, "No s'ha pogut enviar el video.", Toast.LENGTH_LONG).show();
												Log.e(PostVideoActivity.class.getName(), "No s'ha pogut enviar el video - " + e.getMessage());
											}
										}
									});
								} else {
									setResult(RESULT_CANCELED);
									Toast.makeText(PostVideoActivity.this, "No s'ha pogut enviar el video.", Toast.LENGTH_LONG).show();
									Log.e(PostVideoActivity.class.getName(), "No s'ha pogut enviar el video - " + e.getMessage());
								}
							}
						});
					} catch (FileNotFoundException e) {
						setResult(RESULT_CANCELED);
						Toast.makeText(PostVideoActivity.this, "No s'ha pogut recuperar el video de la càmera.", Toast.LENGTH_LONG).show();
						Log.e(PostVideoActivity.class.getName(), "No s'ha pogut recuperar el video de la càmera - " + e.getMessage());
					}
					
					finish();
				}
			});
		} else {
			setResult(RESULT_CANCELED);
			Toast.makeText(this, "No s'ha pogut recuperar el video de la càmera.", Toast.LENGTH_LONG).show();
			Log.e(PostFotoActivity.class.getName(), "No s'ha pogut recuperar el video de la càmera.");
			finish();
		}
	}
	
	/**
	 * Set up the {@link android.app.ActionBar}.
	 */
	private void setupActionBar() {
		getActionBar().setDisplayHomeAsUpEnabled(true);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		// getMenuInflater().inflate(R.menu.post_video, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case android.R.id.home:
			// This ID represents the Home or Up button. In the case of this
			// activity, the Up button is shown. Use NavUtils to allow users
			// to navigate up one level in the application structure. For
			// more details, see the Navigation pattern on Android Design:
			//
			// http://developer.android.com/design/patterns/navigation.html#up-vs-back
			//
			NavUtils.navigateUpFromSameTask(this);
			return true;
		}
		return super.onOptionsItemSelected(item);
	}

	@Override
	public void onClick(View v) {
		// TODO Auto-generated method stub

	}
}
