package net.bonotown.android.aravadebo;

import com.parse.LogInCallback;
import com.parse.ParseException;
import com.parse.ParseUser;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.EditText;
import android.widget.Toast;

public class UsuariEntradaActivity extends Activity {

	private EditText usuari;
	private EditText contrasenya;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		setContentView(R.layout.activity_usuari_entrada);

		usuari = (EditText) findViewById(R.id.usuari);
		contrasenya = (EditText) findViewById(R.id.contrasenya);

		findViewById(R.id.validar_entrada).setOnClickListener(new View.OnClickListener() {
			public void onClick(View view) {

				// Validem els camps del formulari de registre
				boolean errorValidacioFormulari = false;

				StringBuilder validationErrorMessage = new StringBuilder(getResources().getString(R.string.error_intro));

				if (isEmpty(usuari)) {
					errorValidacioFormulari = true;
					validationErrorMessage.append(getResources().getString(R.string.error_blank_username));
				}

				if (isEmpty(contrasenya)) {
					if (errorValidacioFormulari) {
						validationErrorMessage.append(getResources().getString(R.string.error_join));
					}
					errorValidacioFormulari = true;
					validationErrorMessage.append(getResources().getString(R.string.error_blank_password));
				}
				validationErrorMessage.append(getResources().getString(R.string.error_end));

				// comprovació d'errors en el formulari 
				if (errorValidacioFormulari) {
					Toast.makeText(UsuariEntradaActivity.this, validationErrorMessage.toString(), Toast.LENGTH_LONG).show();
					return;
				}

				// inicialitzem el diàleg de progrés
				final ProgressDialog dlg = new ProgressDialog(UsuariEntradaActivity.this);
				dlg.setTitle(getResources().getString(R.string.title_progress));
				dlg.setMessage(getResources().getString(R.string.login_progress_signing_up));
				dlg.show();
				
				//Entrada en rerefons per executar codi un cop finalitzat el procés
				ParseUser.logInInBackground(usuari.getText().toString(),
						contrasenya.getText().toString(), new LogInCallback() {

					@Override
					public void done(ParseUser user, ParseException e) {
						dlg.dismiss();
						if (e != null) {
							// si hi ha hagut un error durant l'entrada mostrem el missatge
							Toast.makeText(UsuariEntradaActivity.this, e.getMessage(), Toast.LENGTH_LONG).show();
						} else {
							// Si no hi ha error preparem la pantalla principal
							Intent intent = new Intent(UsuariEntradaActivity.this, MainActivity.class);
							intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK);
							startActivity(intent); //iniciem l'activitat
							finish(); //finalitzem aquesta activitat
						}
					}
				});
			}
		});
	}

	private boolean isEmpty(EditText etText) {
		if (etText.getText().toString().trim().length() > 0) {
			return false;
		} else {
			return true;
		}
	}
}
