package net.bonotown.android.aravadebo;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;
import android.widget.EditText;
import android.widget.Toast;

import com.parse.ParseException;
import com.parse.ParseUser;
import com.parse.SignUpCallback;

/**
 * Activity which displays a login screen to the user.
 */
public class UsuariRegistreActivity extends Activity {
	private EditText usuari;
	private EditText contrasenya;
	private EditText repeticioContrasenya;
	private EditText email;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		setContentView(R.layout.activity_usuari_registre);

		// Camps del formulari inicialitzats
		usuari = (EditText) findViewById(R.id.usuari_registre);
		contrasenya = (EditText) findViewById(R.id.contrasenya_registre);
		repeticioContrasenya = (EditText) findViewById(R.id.repeticio_contrasenya);
		email = (EditText) findViewById(R.id.email_registre);

		// Captura (handler) del submit del formulari
		findViewById(R.id.validar_registre).setOnClickListener(new View.OnClickListener() {
			public void onClick(View view) {

				// Validem els camps del formulari de registre
				boolean errorValidacioFormulari = false;
				
				StringBuilder validationErrorMessage = new StringBuilder(getResources().getString(R.string.error_intro));
								
				if (isEmpty(usuari)) {
					errorValidacioFormulari = true;
					validationErrorMessage.append(getResources().getString(R.string.error_blank_username));
				}
				
				if (isEmpty(contrasenya)) {
					if (errorValidacioFormulari) {
						validationErrorMessage.append(getResources().getString(R.string.error_join));
					}
					errorValidacioFormulari = true;
					validationErrorMessage.append(getResources().getString(R.string.error_blank_password));
				}
				
				if (!isMatching(contrasenya, repeticioContrasenya)) {
					if (errorValidacioFormulari) {
						validationErrorMessage.append(getResources().getString(R.string.error_join));
					}
					errorValidacioFormulari = true;
					validationErrorMessage.append(getResources().getString(R.string.error_mismatched_passwords));
				}
				
				if (isEmpty(email)) {
					if (errorValidacioFormulari) {
						validationErrorMessage.append(getResources().getString(R.string.error_email_en_blanc));
					}
					errorValidacioFormulari = true;
					validationErrorMessage.append(getResources().getString(R.string.error_email_en_blanc));
				}
				
				validationErrorMessage.append(getResources().getString(R.string.error_end));

				// Si hi ha error per mostrar
				if (errorValidacioFormulari) {
					Toast.makeText(UsuariRegistreActivity.this, validationErrorMessage.toString(), Toast.LENGTH_LONG).show();
					return;
				}

				// inicialitzem el diàleg de progrés
				final ProgressDialog dlg = new ProgressDialog(UsuariRegistreActivity.this);
				dlg.setTitle(getResources().getString(R.string.title_progress));
				dlg.setMessage(getResources().getString(R.string.login_progress_signing_up));
				dlg.show();

				//creem un nou objecte usuari
				ParseUser user = new ParseUser();
				
				//camps per defecte de l'entitat User de parse.com
				user.setUsername(usuari.getText().toString());
				user.setPassword(contrasenya.getText().toString());
				user.setEmail(email.getText().toString());
				
				// Fem el registre en rerefons per poder executar codi un cop acabat
				user.signUpInBackground(new SignUpCallback() {

					@Override
					public void done(ParseException e) {
						dlg.dismiss();
						if (e != null) {
							// Mostrem missatge d'error si n'hi ha hagut
							Toast.makeText(UsuariRegistreActivity.this, e.getMessage(), Toast.LENGTH_LONG).show();
						} else {
							// Engeguem l'activitat principal amb un intent
							Intent intent = new Intent(UsuariRegistreActivity.this, MainActivity.class);
							intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK);
							startActivity(intent); //iniciem a l'activitat
							finish(); //finalitzem aquesta activitat
						}
					}
				});
			}
		});
	}

	private boolean isEmpty(EditText etText) {
		if (etText.getText().toString().trim().length() > 0) {
			return false;
		} else {
			return true;
		}
	}

	private boolean isMatching(EditText etText1, EditText etText2) {
		if (etText1.getText().toString().equals(etText2.getText().toString())) {
			return true;
		} else {
			return false;
		}
	}

}
