package net.bonotown.android.aravadebo.adapter;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;

import com.parse.ParseException;
import com.parse.ParseQuery;

import net.bonotown.android.aravadebo.FragmentAgenda;
import net.bonotown.android.aravadebo.R;
import net.bonotown.android.aravadebo.model.gestio.ItemLlistaAgenda;
import android.content.Context;
import android.graphics.Color;
import android.graphics.Typeface;
import android.support.v4.app.Fragment;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.view.View.OnClickListener;
import android.widget.BaseAdapter;
import android.widget.Button;

public class GridViewCalendariAdapter extends BaseAdapter implements OnClickListener {
	public static final String NOM_MES = "Juny";

	private static final String SPLITTER_DATA = "/";

	private Context context;
	private Fragment parentFragment;
	private List<String> list;
	private int diesMesSeleccionat = 30;
	private Button botoDia;
	
	private HashMap<String, List<ItemLlistaAgenda>> esdevenimentsMesAnySeleccionat;
	
	public GridViewCalendariAdapter(Context context, Fragment fragment, int textViewResourceId, int mes, int any) throws java.text.ParseException {
		super();
		
		this.context = context;
		this.parentFragment = fragment;
		this.list = new ArrayList<String>();

		try {
			esdevenimentsMesAnySeleccionat = recuperarEsdevenimentsMesAny(any, mes);
		} catch (ParseException e) {
			Log.e(context.getClass().getName(), "Problemes en recuperar esdeveniments agenda - " + e.getMessage());
		}
		
		mostrarMesCalendari(mes, any);
	}

	public String getItem(int position) {
		return list.get(position);
	}

	@Override
	public int getCount() {
		return list.size();
	}

	private void mostrarMesCalendari(int mes, int any) throws java.text.ParseException {
		for (int i = 1; i <= diesMesSeleccionat; i++) {
			String data_str = String.valueOf(i) + SPLITTER_DATA + Integer.toString(mes) + SPLITTER_DATA + any;
			
			// comprovar quin dia de la setmana és per començar a partir d'allà
			if (i==1) {
				GregorianCalendar cal = new GregorianCalendar();
				SimpleDateFormat format1 = new SimpleDateFormat("d/M/yyyy");
				Date data = format1.parse(data_str);
				cal.setTime(data);
				
				int dia_setmana = cal.get(Calendar.DAY_OF_WEEK);
				dia_setmana = 7 - dia_setmana;
				
				//afegim tantes entrades en blanc fins el dia que correspon
				for (int dia = 1; dia<= dia_setmana; dia++) {
					list.add("");
				}
			}
			
			list.add(data_str);
		}
	}
	
	// recupera tots els esdeveniments de tots els dies del mes seleccionar i els agrupa per data
	private HashMap<String, List<ItemLlistaAgenda>> recuperarEsdevenimentsMesAny(int any, int mes) throws ParseException {
		final HashMap<String, List<ItemLlistaAgenda>> esdeveniments = new HashMap<String, List<ItemLlistaAgenda>>();

		ParseQuery<ItemLlistaAgenda> query = ItemLlistaAgenda.getQuery();
		query.orderByAscending("data");
		List<ItemLlistaAgenda> llista_items_agenda = query.find();

		if (llista_items_agenda.size()>0) {
			List<ItemLlistaAgenda> esdevenimentsDia = new ArrayList<ItemLlistaAgenda>();;
			String dataActual = "";

			for (ItemLlistaAgenda esdeveniment : llista_items_agenda) {
				if (dataActual!=esdeveniment.getData() && dataActual!="") {
					esdeveniments.put(dataActual, esdevenimentsDia);
					esdevenimentsDia = new ArrayList<ItemLlistaAgenda>();
				}
				esdevenimentsDia.add(esdeveniment);
				dataActual = esdeveniment.getData();
			}

			esdeveniments.put(dataActual, esdevenimentsDia);
		} else {
			Log.i(context.getClass().getName(), "No hi ha esdeveniments a parse.com");
		}

		return esdeveniments;
	}

	@Override
	public long getItemId(int position) {
		return position;
	}

	@Override
	public View getView(int position, View convertView, ViewGroup parent) {

		if (convertView == null) {
			LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
			convertView = inflater.inflate(R.layout.list_item_calendari, parent, false);
		}

		botoDia = (Button) convertView.findViewById(R.id.dia_calendari);
		botoDia.setOnClickListener(this);

		String[] partsData = list.get(position).split(SPLITTER_DATA);
		String dia = partsData[0];
		
		botoDia.setText(dia);
		botoDia.setTag(list.get(position));
		botoDia.setTextColor((Color.RED)); // color vermell en el dia per defecte
		
		if ((!esdevenimentsMesAnySeleccionat.isEmpty()) && (esdevenimentsMesAnySeleccionat != null)) {
			//si el dia té esdeveniments el marquem
			if (esdevenimentsMesAnySeleccionat.containsKey(list.get(position))) {
				botoDia.setTextColor(Color.BLACK); // color negre
				botoDia.setTypeface(null, Typeface.BOLD); // en negreta
				botoDia.setBackground(context.getResources().getDrawable(R.drawable.bandera_st_joan)); //bandera de fons
			}
		}
		
		return convertView;
	}
	
	@Override
	public void onClick(View view) {
		List<ItemLlistaAgenda> list = new ArrayList<ItemLlistaAgenda>();
		String data = (String)view.getTag();
		
		// quan es clica un dia, comuniquem amb el fragment per mostrar els esdeveniments a la llista
		if ((!esdevenimentsMesAnySeleccionat.isEmpty()) && (esdevenimentsMesAnySeleccionat != null)) {
			if (esdevenimentsMesAnySeleccionat.containsKey(data)) {
				list = esdevenimentsMesAnySeleccionat.get(data);				
			}
		}
		
		((FragmentAgenda)parentFragment).mostrarEsdevenimentsDia(list, data);
	}
}