package net.bonotown.android.aravadebo.adapter;

import java.util.ArrayList;

import com.parse.ParseAnonymousUtils;
import com.parse.ParseUser;

import android.app.Activity;
import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.TextView;
import net.bonotown.android.aravadebo.model.NavDrawerButtons;
import net.bonotown.android.aravadebo.model.NavDrawerButtonsAuthUser;
import net.bonotown.android.aravadebo.model.NavDrawerItem;
import net.bonotown.android.aravadebo.R;

public class NavDrawerListAdapter extends BaseAdapter {

	private Context context;
	private ArrayList<Object> navDrawerItems;

	public NavDrawerListAdapter(Context context, ArrayList<Object> navDrawerItems) {
		this.context = context;
		this.navDrawerItems = navDrawerItems;
	}
	
	@Override
	public int getCount() {
		return navDrawerItems.size();
	}

	@Override
	public Object getItem(int position) {
		return navDrawerItems.get(position);
	}

	@Override
	public long getItemId(int position) {
		return position;
	}

	//valor 1 pels botons de registre i 0 pels items de menú principal
	@Override
	public int getItemViewType(int position) {
		return (((getItem(position) instanceof NavDrawerButtons) || 
				(getItem(position) instanceof NavDrawerButtonsAuthUser)) ? 1 : 0);
	}

	@Override
	public View getView(int position, View convertView, ViewGroup parent) {
		
		if (getItemViewType(position)==0) { // si són ítems del menú principal
			if (convertView == null) {
				LayoutInflater mInflater = (LayoutInflater) context.getSystemService(Activity.LAYOUT_INFLATER_SERVICE);
				convertView = mInflater.inflate(R.layout.drawer_list_item, null);
			}

			TextView txtTitle = (TextView) convertView.findViewById(R.id.title);
			txtTitle.setText(((NavDrawerItem)(navDrawerItems.get(position))).getTitle());
		} else { // si són botons d'accions d'usuari
			//Comprovem si l'usuari és anònim
			if (! ParseAnonymousUtils.isLinked(ParseUser.getCurrentUser())) {
				if (convertView == null) {
					LayoutInflater mInflater = (LayoutInflater) context.getSystemService(Activity.LAYOUT_INFLATER_SERVICE);
					convertView = mInflater.inflate(R.layout.drawer_list_button_item_auth_user, null);
				}
				
				//Botons per usuari autenticat
				NavDrawerButtonsAuthUser navDrawerButtonsAuthUser = new NavDrawerButtonsAuthUser(context);
				
				//botó per deconnectar usuari
				navDrawerButtonsAuthUser.setBotoTancar((Button) convertView.findViewById(R.id.boto_tancar));
				navDrawerButtonsAuthUser.getBotoTancar().setOnClickListener(navDrawerButtonsAuthUser.clickTancarListener);
				
				//botó per mostrar pantalla d'ajuda
				navDrawerButtonsAuthUser.setBotoAjuda((Button) convertView.findViewById(R.id.boto_ajuda));
				navDrawerButtonsAuthUser.getBotoAjuda().setOnClickListener(navDrawerButtonsAuthUser.clickAjudaListener);
			} else {
				if (convertView == null) {
					LayoutInflater mInflater = (LayoutInflater) context.getSystemService(Activity.LAYOUT_INFLATER_SERVICE);
					convertView = mInflater.inflate(R.layout.drawer_list_button_item, null);
				}
				
				//botons per usuaris anònims
				NavDrawerButtons navDrawerButtons = new NavDrawerButtons(context);
				
				//botó per entrar a l'aplicació
				navDrawerButtons.setBotoEntrar((Button) convertView.findViewById(R.id.boto_entrar));
				navDrawerButtons.getBotoEntrar().setOnClickListener(navDrawerButtons.clickEntrarListener);
				
				//botó per registrar-se a l'aplicació
				navDrawerButtons.setBotoRegistrar((Button) convertView.findViewById(R.id.boto_registrar));
				navDrawerButtons.getBotoRegistrar().setOnClickListener(navDrawerButtons.clickRegistrarListener);
			}
		}

		return convertView;
	}

	@Override
	public int getViewTypeCount() {
		return 2; //indiquem que hi ha dos tipus d'objectes en aquest drawer
	}
}